from gi.repository import Gtk, Handy
from whatip.confManager import ConfManager
from whatip.headerbar import GHeaderbar
from whatip.main_stack import MainStack


class AppWindow(Handy.ApplicationWindow):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.confman = ConfManager()

        self.set_title('What IP')
        self.set_icon_name('org.gabmus.whatip')

        self.main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.main_stack = MainStack()
        self.main_stack.connect(
            'notify::visible-child',
            self.on_main_stack_change
        )
        self.bottom_bar = Handy.ViewSwitcherBar()
        self.bottom_bar.set_stack(self.main_stack)
        
        self.iface_listbox = self.main_stack.ip_listbox
        self.headerbar = GHeaderbar(self.main_stack)
        self.on_main_stack_change()  # sets text to current stack child
        self.headerbar.connect('headerbar_squeeze', self.on_headerbar_squeeze)

        self.main_box.add(self.headerbar)
        self.headerbar.set_vexpand(False)
        self.main_box.add(self.main_stack)
        self.main_stack.set_vexpand(True)
        self.main_box.add(self.bottom_bar)
        self.bottom_bar.set_vexpand(False)

        self.add(self.main_box)

        # Why this -52?
        # because every time a new value is saved, for some reason
        # it's the actual value +52 out of nowhere
        # this makes the window ACTUALLY preserve its old size
        self.resize(
            self.confman.conf['windowsize']['width']-52,
            self.confman.conf['windowsize']['height']-52
        )
        self.size_allocation = self.get_allocation()
        self.connect('size-allocate', self.update_size_allocation)

        # accel_group is for keyboard shortcuts
        self.accel_group = Gtk.AccelGroup()
        self.add_accel_group(self.accel_group)
        shortcuts_l = [
            {
                'combo': 'F10',
                'cb': lambda *args: self.headerbar.menu_btn.clicked()
            }
        ]
        for s in shortcuts_l:
            self.add_accelerator(s['combo'], s['cb'])

    def on_main_stack_change(self, *args):
        self.headerbar.nobox.set_text(
            self.main_stack.get_visible_child_name()
        )

    def on_headerbar_squeeze(self, caller, squeezed):
        self.bottom_bar.set_reveal(squeezed)

    def add_accelerator(self, shortcut, callback):
        if shortcut:
            key, mod = Gtk.accelerator_parse(shortcut)
            self.accel_group.connect(
                key, mod, Gtk.AccelFlags.VISIBLE, callback
            )

    def emit_destroy(self, *args):
        self.emit('destroy')

    def on_destroy(self, *args):
        self.confman.conf['windowsize'] = {
            'width': self.size_allocation.width,
            'height': self.size_allocation.height
        }
        self.confman.save_conf()

    def update_size_allocation(self, *args):
        self.size_allocation = self.get_allocation()

    def do_startup(self):
        self.headerbar.on_refresh_btn_clicked()
