# from gettext import gettext as _
from gi.repository import Gtk, GObject, Handy
from whatip.confManager import ConfManager


class GHeaderbar(Handy.WindowHandle):
    __gsignals__ = {
        'headerbar_squeeze': (
            GObject.SignalFlags.RUN_FIRST,
            None,
            (bool,)
        )
    }

    def __init__(self, main_stack, **kwargs):
        super().__init__(**kwargs)
        self.main_stack = main_stack
        self.builder = Gtk.Builder.new_from_resource(
            '/org/gabmus/whatip/ui/headerbar.glade'
        )
        self.builder.connect_signals(self)
        self.confman = ConfManager()
        self.headerbar = self.builder.get_object('headerbar')
        self.squeezer = Handy.Squeezer(orientation=Gtk.Orientation.HORIZONTAL)
        self.squeezer.set_homogeneous(False)
        self.squeezer.set_interpolate_size(False)
        self.squeezer.set_hexpand(False)
        self.nobox = Gtk.Label()
        self.nobox.get_style_context().add_class('title')
        self.view_switcher = Handy.ViewSwitcher()
        self.view_switcher.set_stack(self.main_stack)
        self.view_switcher.set_margin_start(12)
        self.view_switcher.set_margin_end(12)
        self.view_switcher.set_policy(Handy.ViewSwitcherPolicy.WIDE)
        self.squeezer.add(self.view_switcher)
        self.squeezer.add(self.nobox)
        self.squeezer.connect('notify::visible-child', self.on_squeeze)
        self.headerbar.set_custom_title(self.squeezer)

        self.add(self.headerbar)
        self.menu_btn = self.builder.get_object(
            'menu_btn'
        )
        self.refresh_btn = self.builder.get_object(
            'refresh_btn'
        )
        self.menu_popover = Gtk.PopoverMenu()
        self.menu_btn.set_popover(self.menu_popover)
        self.menu_builder = Gtk.Builder.new_from_resource(
            '/org/gabmus/whatip/ui/menu.xml'
        )
        self.menu = self.menu_builder.get_object('generalMenu')
        self.menu_popover.bind_model(self.menu)
        self.set_headerbar_controls()
        self.main_stack.connect('refresh_finished', self.on_refresh_finished)

    def on_squeeze(self, *args):
        self.emit(
            'headerbar_squeeze',
            self.squeezer.get_visible_child() == self.nobox
        )

    def set_headerbar_controls(self, *args):
        self.headerbar.set_show_close_button(True)
        self.headerbar.set_title('What IP')

    def on_refresh_btn_clicked(self, *args, sync=False):
        self.refresh_btn.set_sensitive(False)
        self.main_stack.ip_listbox.refresh()
        self.main_stack.port_listbox.refresh()
        self.main_stack.populate_lan_listbox()

    def on_refresh_finished(self, *args):
        self.refresh_btn.set_sensitive(True)
