from gi.repository import Gtk, Gdk, Gio, GLib


class LanListboxRow(Gtk.ListBoxRow):
    def __init__(self, address, hostname, ports, local_address, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.address = address
        self.hostname = hostname
        self.local_address = local_address
        self.builder = Gtk.Builder.new_from_resource(
            '/org/gabmus/whatip/ui/lan_listbox_row.glade'
        )
        self.mainbox = self.builder.get_object(
            'mainbox'
        )
        self.address_label = self.builder.get_object(
            'address_label'
        )
        self.hostname_label = self.builder.get_object(
            'hostname_label'
        )
        self.local_address_label = self.builder.get_object(
            'local_address_label'
        )
        self.go_http_btn = self.builder.get_object(
            'go_http_btn'
        )
        self.go_ftp_btn = self.builder.get_object(
            'go_ftp_btn'
        )
        self.address_label.set_text(self.address)
        self.hostname_label.set_text(self.hostname)
        self.local_address_label.set_text(self.local_address)

        port_interaction_buttons = {
            80: self.go_http_btn,
            21: self.go_ftp_btn
        }
        for port in [80, 21]:
            if port in ports:
                port_interaction_buttons[port].set_no_show_all(False)
                port_interaction_buttons[port].set_visible(True)

        self.add(self.mainbox)

        self.builder.connect_signals(self)

    def on_copy_btn_clicked(self, *args):
        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
        clipboard.set_text(self.address, -1)
        clipboard.store()

    def on_go_http_btn_clicked(self, *args):
        Gtk.show_uri_on_window(None, f'http://{self.address}', Gdk.CURRENT_TIME)

    def on_go_ftp_btn_clicked(self, *args):
        bus = Gio.bus_get_sync(Gio.BusType.SESSION, None)
        file_manager = Gio.DBusProxy.new_sync(bus,
                                             Gio.DBusProxyFlags.NONE,
                                             None,
                                             'org.freedesktop.FileManager1',
                                             '/org/freedesktop/FileManager1',
                                             'org.freedesktop.FileManager1',
                                             None)

        file_manager.call_sync('ShowItems',
                               GLib.Variant(
                                    '(ass)',
                                    ([f'ftp://{self.address}'], '')
                               ),
                               Gio.DBusCallFlags.NONE,
                               -1,
                               None)
