from gi.repository import Gtk, GObject
from gettext import gettext as _
from whatip.network_interface_listbox import NetworkInterfaceListbox
from whatip.port_listbox import PortListbox
from whatip.lan_listbox import LanListbox
from whatip.network_interface import NetworkInterface


class MainStack(Gtk.Stack):
    __gsignals__ = {
        'refresh_finished': (
            GObject.SignalFlags.RUN_LAST,
            None,
            (str,)
        )
    }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.set_transition_type(Gtk.StackTransitionType.CROSSFADE)

        self.ip_listbox = NetworkInterfaceListbox()
        self.add_titled(self.ip_listbox, 'IP', _('IP'))
        self.child_set_property(
            self.ip_listbox,
            'icon-name',
            'network-wired-symbolic'
        )

        self.port_listbox = PortListbox()
        self.add_titled(self.port_listbox, 'Ports', _('Ports'))
        self.child_set_property(
            self.port_listbox,
            'icon-name',
            'network-transmit-receive-symbolic'
        )

        self.lan_listbox = LanListbox()
        self.add_titled(self.lan_listbox, 'LAN', _('LAN'))
        self.child_set_property(
            self.lan_listbox,
            'icon-name',
            'preferences-system-network-symbolic'
        )
        self.lan_listbox.connect(
            'refresh_finished',
            self.on_lan_refresh_finished
        )

        self.show_all()

    def populate_lan_listbox(self):
        iface_l = list(filter(
            lambda iface: iface.interface_type not in (
                NetworkInterface.LOOPBACK,
                NetworkInterface.INTERNET,
                NetworkInterface.TUNNEL,
                NetworkInterface.UNKNOWN
            ),
            self.ip_listbox.iface_l
        ))
        if len(iface_l) <= 0:
            self.on_lan_refresh_finished()
            return
        self.lan_listbox.refresh([
            (iface.address, iface.subnet_mask)
            for iface in iface_l
        ])

    def on_lan_refresh_finished(self, *args):
        self.emit('refresh_finished', '')
