from gi.repository import Gtk
from gettext import gettext as _
from whatip.network_ports import get_ports
from whatip.port_listbox_row import PortListboxRow
from whatip.listbox_with_empty_state import ListBoxWithEmptyState


class PortListbox(ListBoxWithEmptyState):
    def __init__(self, *args, **kwargs):
        super().__init__(
            'network-offline-symbolic',
            _('No ports detected'),
            *args, **kwargs
        )
        self.set_selection_mode(Gtk.SelectionMode.NONE)
        self.set_header_func(self.separator_header_func)

        # self.refresh()

    def separator_header_func(self, row, prev_row=None):
        if (
                prev_row is not None and
                row.get_header() is None
        ):
            row.set_header(
                Gtk.Separator(orientation=Gtk.Orientation.HORIZONTAL)
            )

    def refresh(self):
        self.empty(False)
        ports = get_ports()
        for port in ports.keys():
            self.add(
                PortListboxRow(
                    '\n'.join(ports[port]['addresses']),
                    port,
                    ports[port]['type']
                )
            )
        if len(ports) <= 0:
            self.set_show_empty_state(True)
        else:
            self.show_all()
